#!/bin/bash
BACKUP_PATH=/tmp/backup
mkdir -p $BACKUP_PATH
SERVER_LOG=$BACKUP_PATH/messages.log


# get_conf_variable
#   $1 = config file name
#   $2 = variable name
#   $3 = default value if not found
get_conf_variable() {
	#RET=`awk "/^$2\>/ {print \\$3}" $1`
	RET=`awk "/^$2\>/ {print \\$3}" $1`
	if [ "$RET" == "" ]; then
		echo $3
	else
		echo $RET
	fi
}

# set_conf_variable
#   $1 = config file name
#   $2 = variable name
#   $3 = value
set_conf_variable() {
	TMP_CONF_FILENAME=/tmp/$$.set_conf_variable
	rm -f $TMP_CONF_FILENAME
	WRITTEN=0
	touch $1
	while read LINE; do
		echo $LINE | egrep "^$2\>" > /dev/null
		RESULT=$?
		if [ "$RESULT" == "0" ]; then
			# found
			echo "$2 = $3" >> $TMP_CONF_FILENAME
			WRITTEN=1
		else
			# not found
			echo $LINE >> $TMP_CONF_FILENAME
		fi
	done < $1
	if [ "$WRITTEN" == "0" ]; then
		echo "$2 = $3" >> $TMP_CONF_FILENAME
	fi
	cp -f $TMP_CONF_FILENAME $1
	rm -f $TMP_CONF_FILENAME
}


echo "nanobackup [`date +'%D %H:%M:%S'`] init" >> $SERVER_LOG
read CLIENT_HOST
echo "nanobackup [`date +'%D %H:%M:%S'`] client_host=$CLIENT_HOST received" >> $SERVER_LOG
#echo "Client host = '$CLIENT_HOST'"
read CLIENT_PATH
echo "nanobackup [`date +'%D %H:%M:%S'`] client_path=$CLIENT_PATH received" >> $SERVER_LOG
#echo "Client path = '$CLIENT_PATH'"
read CLIENT_FILENAME_PREFIX
echo "nanobackup [`date +'%D %H:%M:%S'`] client_filename_prefix=$CLIENT_FILENAME_PREFIX received" >> $SERVER_LOG
read CLIENT_FILESIZE
echo "nanobackup [`date +'%D %H:%M:%S'`] client_filesize=$CLIENT_FILESIZE received" >> $SERVER_LOG
#echo "Client filename = '$CLIENT_FILENAME'"
#echo "Client path = '$CLIENT_PATH'"
#echo "Client filename = '$CLIENT_FILENAME'"
mkdir -p $BACKUP_PATH/$CLIENT_HOST
# leemos la configuracin para este cliente
if [ -f $BACKUP_PATH/$CLIENT_HOST/$CLIENT_FILENAME_PREFIX.conf ]; then
	echo "nanobackup [`date +'%D %H:%M:%S'`] $BACKUP_PATH/$CLIENT_HOST/$CLIENT_FILENAME_PREFIX.conf exists. Getting variables" >> $SERVER_LOG
	NEXT_INDEX=`get_conf_variable "$BACKUP_PATH/$CLIENT_HOST/$CLIENT_FILENAME_PREFIX.conf" "next_index" 0`
	NUM_INDICES=`get_conf_variable "$BACKUP_PATH/$CLIENT_HOST/$CLIENT_FILENAME_PREFIX.conf" "num_indices" 7`
else
	if [ -f $BACKUP_PATH/$CLIENT_HOST/default.conf ]; then
		echo "nanobackup [`date +'%D %H:%M:%S'`] $BACKUP_PATH/$CLIENT_HOST/default.conf exists. Getting variables" >> $SERVER_LOG
		NEXT_INDEX=`get_conf_variable "$BACKUP_PATH/$CLIENT_HOST/default.conf" "next_index" 0`
		NUM_INDICES=`get_conf_variable "$BACKUP_PATH/$CLIENT_HOST/default.conf" "num_indices" 7`
	else
		echo "nanobackup [`date +'%D %H:%M:%S'`] $BACKUP_PATH/$CLIENT_HOST/default.conf does not exist. Initializing variables" >> $SERVER_LOG
		NEXT_INDEX=0
		NUM_INDICES=7
	fi
fi
echo "nanobackup [`date +'%D %H:%M:%S'`] next_index=$NEXT_INDEX num_indices=$NUM_INDICES" >> $SERVER_LOG
set_conf_variable "$BACKUP_PATH/$CLIENT_HOST/$CLIENT_FILENAME_PREFIX.conf" "next_index" $((($NEXT_INDEX + 1) % $NUM_INDICES))
set_conf_variable "$BACKUP_PATH/$CLIENT_HOST/$CLIENT_FILENAME_PREFIX.conf" "client_path" $CLIENT_PATH
dd bs=1 count=$CLIENT_FILESIZE > $BACKUP_PATH/$CLIENT_HOST/$NEXT_INDEX.$CLIENT_FILENAME_PREFIX.tar.gz 2> /dev/null
read MD5SUM_CLIENT
echo "nanobackup [`date +'%D %H:%M:%S'`] md5sum_client='$MD5SUM_CLIENT'" >> $SERVER_LOG
MD5SUM_SERVER=`md5sum < $BACKUP_PATH/$CLIENT_HOST/$NEXT_INDEX.$CLIENT_FILENAME_PREFIX.tar.gz | cut -d' ' -f1`
echo "nanobackup [`date +'%D %H:%M:%S'`] md5sum_server='$MD5SUM_SERVER'" >> $SERVER_LOG
if [ "$MD5SUM_CLIENT" == "$MD5SUM_SERVER" ]; then
	echo "OK"
	echo "nanobackup [`date +'%D %H:%M:%S'`] done: ok" >> $SERVER_LOG
	echo $MD5SUM_SERVER > $BACKUP_PATH/$CLIENT_HOST/$NEXT_INDEX.$CLIENT_FILENAME_PREFIX.md5
else
	echo "ERROR"
	echo "nanobackup [`date +'%D %H:%M:%S'`] done: md5sum error" >> $SERVER_LOG
	echo "ERROR" > $BACKUP_PATH/$CLIENT_HOST/$NEXT_INDEX.$CLIENT_FILENAME_PREFIX.md5
fi
